<?php

class MPP_Gallery_Query extends WP_Query{
    private $post_type;
    
    public function __construct( $query = '' ) {
        
        $this->post_type = mpp_get_gallery_post_type(); 
        
        parent::__construct( $query );
        
	}
    
    public function query( $args ){
                //make sure that the query params was not built before
        if( !isset( $args['_mpp_mapped_query'] ) )
            $args = self::build_params( $args );
        parent::query( $args  );
        
    }
    //map gallery parameters to wp_query parameters
    public function build_params( $args ){
        
        $defaults = array(
                'type'          => false, //gallery type, all,audio,video,photo etc
                'id'            => false, //pass specific gallery id
                'in'            => false, //pass specific gallery ids as array
                'exclude'       => false, //pass gallery ids to exclude
                'slug'          => false,//pass gallery slug to include
                'status'        => false, //public,private,friends one or more privacy level
                'component'     => false, //one or more component name user,groups, evenets etc
                'component_id'  => false,// the associated component id, could be group id, user id, event id
                'per_page'      => mpp_get_option( 'galleries_per_page' ),
                'offset'        => false, //how many galleries to offset/displace
                'page'          => false,//which page when paged
                'nopaging'      => false, //to avoid paging
                'order'         => 'DESC',//order 
                'orderby'       => 'date',//none, id, user, title, slug, date,modified, random, comment_count, meta_value,meta_value_num, ids
                //user params
                'user_id'       => false,
                'include_users' => false,
                'exclude_users' => false,//users to exclude
                'user_name'     => false,
                'scope'         => false,
                'search_terms'  => '',
            //time parameter
                'year'          => false,//this years
                'month'         => false,//1-12 month number
                'week'          => '', //1-53 week
                'day'           => '',//specific day
                'hour'          => '',//specific hour
                'minute'        => '', //specific minute
                'second'        => '',//specific second 0-60
                'yearmonth'     => false,// yearMonth, 201307//july 2013
                //'meta_key'=>'',
               // 'meta_value'=>'',
               // 'meta_query'=>false,
                'fields'    => false,//which fields to return ids, id=>parent, all fields(default)
        );
        
     
        //build params for WP_Query
        
        
    $r = wp_parse_args( $args, $defaults );
    extract( $r, EXTR_SKIP );
    
    
    //build the wp_query args
    $wp_query_args = array(
        'post_type'         => mpp_get_gallery_post_type(),
        'post_status'       => 'any',
        'p'                 => $id,
        'post__in'          => $in,
        'post__not_in'      => $exclude,
        'name'              => $slug,
        'posts_per_page'    => $per_page,
        'paged'             => $page,
        'offset'            => $offset,
        'nopaging'          => $nopaging,
        //user params
        'author'            => $user_id,
        'author_name'       => $user_name,
        'author__in'        => $include_users,
        'author__not_in'    => $exclude_users,
        
        //date time params
        
        'year'              => $year,
        'monthnum'          => $month,
        'w'                 => $week,
        'day'               => $day,
        'hour'              => $hour,
        'minute'            => $minute,
        'second'            => $second,
        'm'                 => $yearmonth,
        //order by
        'order'             => $order,
        'orderby'           => $orderby,
        's'                 => $search_terms,
        //meta key, may be we can set them here?
       // 'meta_key'=>$meta_key,
        //'meta_value'=>$meta_value,
        //which fields to fetch
        'fields'            => $fields,
        '_mpp_mapped_query' => true,
        
    );
    
    $tax_query = array();
    $gmeta_query = array();//meta
    
    if( isset($meta_key) && $meta_key )
        $wp_query_args['meta_key'] = $meta_key;
    
    if( isset( $meta_value ) )
        $wp_query_args['meta_value'] = $meta_value;
    
    if( isset( $meta_query ))
        $gmeta_query = $meta_query;
    //TODO: SCOPE
    //
    
    
    //we will need to build tax query/meta query
    
    //type, audio video etc
    //if type is given and it is valid gallery type
    //Pass one or more types
    if( !empty( $type ) && mpp_are_registered_gallery_types( $type ) ){
        
        $type = mpp_string_to_array( $type ); 
        $type = array_map( 'mpp_underscore_it', $type );
        
        $tax_query[] = array(
                'taxonomy' => mpp_get_type_taxname(),
                'field'=> 'slug',
                'terms'=> $type,
                'operator'=> 'IN',
        );
    }
    
    //privacy
    //pass ne or more privacy level
    if( !empty( $status ) && mpp_are_registered_gallery_statuses( $status ) ){
        
        $status = mpp_string_to_array( $status );
        $status = array_map( 'mpp_underscore_it', $status );
        
        $tax_query[] = array(
                'taxonomy'  => mpp_get_status_taxname(),
                'field'     => 'slug',
                'terms'     => $status,
                'operator'  => 'IN'
        ); 
    }
    
    if( !empty ( $component ) && mpp_are_valid_gallery_components( $component ) ){
        
        $component = mpp_string_to_array( $component ); 
        $component = array_map( 'mpp_underscore_it', $component );
        
        $tax_query[] = array(
                'taxonomy'  => mpp_get_component_taxname(),
                'field'     => 'slug',
                'terms'     => $component,
                'operator'  => 'IN'
        ); 
       
    }
    
    //done with the tax query
    
    if( count( $tax_query ) > 1 ){
        
        $tax_query['relation'] = 'AND';
       
    }
    
    $wp_query_args['tax_query'] = $tax_query;
   // print_nice($wp_query_args);
    //meta query
   
   //now, for component
    if( !empty( $component_id ) ){
     
        $meta_compare = '=';
    
        if( is_array( $component_id ) )
            $meta_compare = 'IN';
    
        $gmeta_query[] = array(
            'key'       => '_mpp_component_id',
            'value'     => $component_id,
            'compare'   => $meta_compare,
            'type'      => 'UNSIGNED'

        );
    
    }
    
    //reset meta query
    if( !empty( $gmeta_query ) ){
        
        $wp_query_args['meta_query'] = $gmeta_query;
    }
    
   // print_r($wp_query_args);
    return $wp_query_args;
   
   
    
    }
    
       
    public function get_galleries(){
        
        return parent::get_posts();
    }
    

    public function next_gallery(){
        
        return parent::next_post();
        
    }

    
    public function the_gallery() {
                
        global $post;
		$this->in_the_loop = true;

           
         if ( $this->current_post == -1 ) // loop has just started
                do_action_ref_array( 'mediapress_gallery_loop_start', array(&$this));

		$post = $this->next_gallery();
		
        
        setup_postdata( $post );
         
        mediapress()->current_gallery = mpp_get_gallery( $post->ID );
        mpp_setup_gallery_data( $post );
       
		
       
    }
    
    public function have_galleries(){
        
        return parent::have_posts();
    }
    
    public function rewind_galleries() {
        parent::rewind_posts();
    }
    
    
    public function is_main_query(){

        $mediappress = mediapress();
        
        return $this == $mediappress->the_gallery_query;

    }
    
    
	function reset_gallery_data() {
        parent::reset_postdata();
		if ( ! empty( $this->post ) ) {
			mediapress()->current_gallery = mpp_get_gallery( $this->post->ID );
			
		}
	}
    
    /**
     * Putting helpers to allow easy pagination in the loops
     */
    public function paginate(){
        
        $total = $this->max_num_pages;
                // only bother with the rest if we have more than 1 page!
        if ( $total > 1 )  {
            // get the current page
            if ( !$current_page = $this->get('paged') )
                    $current_page = 1;
            // structure of “format” depends on whether we’re using pretty permalinks
            $perma_struct=get_option('permalink_structure');
            $format = empty( $perma_struct ) ? '&page=%#%' : 'page/%#%/';
     
     
    
            $link=  get_pagenum_link(1) ;
     
    
            //$format=$format;
            $base = $link;
           return paginate_links(array(
                 'base' => $base.'%_%',
                 'format' => $format,
                 'current' => $current_page,
                 'total' => $total,
                 'mid_size' => 4,
                 'type' => 'list'
            ));
        }
    }
      
    
    public function pagination_count(){
            
        
        $paged = $this->get( 'paged' )? $this->get( 'paged' ) : 1;
        $posts_pet_page = $this->get( 'posts_per_page' );

        $from_num = intval( ( $paged - 1 ) * $posts_pet_page ) + 1;

        $to_num = ( $from_num + ( $posts_pet_page - 1 ) > $this->found_posts ) ? $this->found_posts : $from_num + ( $posts_pet_page - 1) ;

        echo sprintf( __( 'Viewing gallery %d to %d (of %d galleries)', 'mediapress' ), $from_num, $to_num, $this->found_posts ); 
    }
    
	/**
	 * Get all the ids in this request
	 */
	public function get_ids(){
		$ids = array();
		
		if( empty( $this->request ) )
			return $ids;
		global $wpdb;
		$ids = $wpdb->get_col( $this->request);
		return $ids;
	}
}//end of class

function mpp_setup_gallery_data( $post ) {
	
	//setup gallery data for current gallery 

	return true;
}
/**
 * Reset global gallery data
 */
function mpp_reset_gallery_data(){
    
    mediapress()->the_gallery_query->reset_gallery_data();
    wp_reset_postdata();
}